/** 
 *   FIXME:
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#ifndef _ANTENNA_IB_PK_H
#define _ANTENNA_IB_PK_H

#include <stdint.h>
#include <endian.h>
#include <limits.h>

#include "iar_bus.h"

#define USE_ANTENNA_LOCAL 0

#define ANTENNA_TCP_PORT 2020
#define DWT_TCP_PORT     2021

#define ANTENNA_HOST_NAME "163.10.43.51"
//#define ANTENNA_HOST_NAME "127.0.0.1"
#define DWT_HOST_NAME     "163.10.43.3"
//#define DWT_HOST_NAME     "127.0.0.1"

/* Internal comunication: */
#define ANTENNA_SOCK_NAME "/tmp/antenna_track" 

#define ENCODERS_BUS_ADDR 0x001
#define SURVEY_BUS_ADDR   0x002
#define DWT_BUS_ADDR      0x003

extern char antenna_hostname[];
extern char dwt_hostname[];

#if USE_ANTENNA_LOCAL
#  define ANTENNA_BUS_START() iar_bus_start(ANTENNA_SOCK_NAME)
#  define ANTENNA_CALL_SOCKET() call_socket_local(ANTENNA_SOCK_NAME)
#else
#  define ANTENNA_BUS_START() iar_bus_inet_start(ANTENNA_TCP_PORT)
#  define ANTENNA_CALL_SOCKET() call_socket(antenna_hostname, ANTENNA_TCP_PORT)
#endif

#define DWT_BUS_START() iar_bus_inet_start(DWT_TCP_PORT)
#define DWT_CALL_SOCKET() call_socket(dwt_hostname, DWT_TCP_PORT)


struct antenna_packet_head {
#if BYTE_ORDER == BIG_ENDIAN
	uint32_t cmd:8;
	uint32_t size:24;
#endif
#if BYTE_ORDER == LITTLE_ENDIAN || BYTE_ORDER == PDP_ENDIAN
	uint32_t size:24;
	uint32_t cmd:8;
#endif
};


#define ANTENA_PACKET_DATA_LEN ( (int) IB_PACKET_DATA_LEN - (int) sizeof(struct antenna_packet_head))


struct antenna_packet{
        struct antenna_packet_head h;
        unsigned char data[ANTENA_PACKET_DATA_LEN];
};

/* Encoders command: */
#define ECM_GETBUFFLEN     0x01
#define ECM_CLEARBUFF      0x02
#define ECM_START          0x03
#define ECM_STOP           0x04
#define ECM_ISRUNNING      0x05
#define ECM_GETNVAL        0x06
#define ECM_SETCW          0x07
#define ECM_SETCCW         0x08
#define ECM_SETZERO        0x09
/* Sidereal clock is running? */
#define ECM_SCLKRUNNING    0x0a
/* Permit to get position on Sidereal clock fail condition */
#define ECM_FAILMODE       0x0b
/* Restore to normal condition operation */ 
#define ECM_NORMALMODE     0x0c
#define ECM_GETBITRES      0x0d
#define ECM_POSITION       0x0e

/* Survey command: */
#define SCM_INFO           0x0f
#define SCM_SURVEY_START   0x10
#define SCM_SURVEY_STOP    0x11
#define SCM_SURVEY_STATUS  0x12

/* DWT command: */
#define WCM_LOOP           0x14
#define WCM_SETTIME        0x15
#define WCM_GETTIME        0x16
#define WCM_TEST           0x17
#define WCM_RXCHECK        0x18
#define WCM_VER            0x19
#define WCM_WRD            0x1a

#define WCM_WIND           0x1b

/* Global command: */
#define CM_RETURN          0xff



#endif  /* ifndef _ANTENNA_IB_PK_H */

